close all
clear all
clc


%% OSNOVNE OPERACIJE MATRICNE ALGEBRE


%     A+B      zbrajanje matrica iste dimenzije
%     A*B      monzenje (matricno)
%     A^p      potenciranje (matricno)
%     A'       konjugirano transponirana matrica
%     A.'      transponirana matrica
% ==, >, <     relacije

A = [1 2 3; 4 5 6]
B = [1 -2 7; 4 5 11]
A==B
A>B

A+B
A-B

c = 5
c*A

A = [1 2 3; 4 5 6]
B = [1 1 1; -1 -1 -1; 1/3 1/3 1/3]'
A*B

A = [1+1i 2-2*1i; -1i 4]
A'
A.'

% Primjer: pokazite da je matrica nilpotentna.
M = [0 2 1; 0 0 -3; 0 0 0]
M^2
M^3

% KREIRANJE "POSEBNIH" VRSTA MATRICA 

M1 = zeros(2,5) % nul matrica dimanzija 2 x 5

M2 = ones(2,3) % matrica dimenzija 2 x 3 kojoj su svi elementi jednaki 1

M3 = diag([1 2 3]) % dijagonalna matrica

M4 = eye(5) % jedinicna matrica dimenzija 5 x 5

M5 = blkdiag(M2, M3, M4) % blok dijagonalna matrica



%% SKALARNI I VEKTORSKI PRODUKT

% skalarni produkt
x = [1; 2; 3]
y = [3; -2; 4]
dot(x, y)
x'*y

% provjeriti da li su vektori okomiti
x = [3; 1; 4]
y = [-1; 1; 0.5]
dot(x, y)

% kut izmedju sva vektora
a = [1;1];
b = [1;0];
kut = acos((a'*b)/(norm(a, 2)*norm(b, 2)))
pi/4

% vektorski produkt
v = [1; 0; 0]
w = [0; 1; 0]
cross(v, w)
cross(w, v)


% provjerite da li su vektori paralelni
v = [3; -2; 5]
w = [-1.5; 1; -2.5]
cross(v, w)



%% DETERMINANTA, RANG, INVERZ I TRAG MATRICE

% Determinanta matrice
A = [-1 1; 2 5]
det(A)

% Inverz matrice
inv(A)
A^-1
A\eye(size(A))

% Rang matrice
rank(A)

% Detarminanta dijagonalne matrice je jednaka umnosku elemenata na dijagonali
A = diag([1;2;3])
det(A)  

% Determinanta je nula: matrica je singularna (4. redak je zbroj prva dva - linearno zavisni)
A = [1 2 3; 3 2 1; 4 4 4]
det(A)           

% Trag matrice: zbroj elemenata na dijagonali matrice
A = [1 2 3; 4 5 6; 7 8 9]
trace(A)



%% SUSTAVI LINEARNIH ALGEBARSKIH JEDNADZBI

% Rjesavanje sustava A*x=b

%%%  1) A je kvadratna matrica n x n %%%

A = [-2 3 1;2 2 -1;1 3 2]
b = [1;2;3]

x = A\b % rjesenje sustava Ax=b, tj. x=A\b

% provjera:
r = A*x - b

% rjesenje koristeci inverznu matricu: inv(A)
x1 = inv(A)*b % numericki je bolje koristiti A\b nego inv(A)*b

r1 = A*x1 - b % uocite da je r1 razlicito od gore izracunatog r 
              % (drugim numerickim putem su dobiveni rezultati)      


%%% 2) A je matrica n x m gdje je n>m, broj redaka > broj stupaca %%%
% eng. "overdetermined system"
% najcesce jednadzbe nemaju rjesenje (n jednadzbi, m nepoznanica)
% operator \ daje "least square solution", tj. takav vektor x za koji je 
% vrijednost ||b-A*x||-->min, gdje || . || oznacava euklidsku normu vektora (p=2 normu, vidi dolje) 

%%%%%%%
% primjer a)

A = [1 0; 0 1; 0 0]
b = [0.5;0.5;0.5]

% uocite da vektor b ne moze biti prikazan kao linearna kobimbinacija
% vektora a1 i a2, gdje je a1 prvi stupac iz matrice A, a a2 je drugi
% stupac iz matrice A. To znaci da vektor b nije u slici od matrice A, sto
% dalje znaci da jednadzba Ax=b nema rjesenja. Ipak MATLAB ce nam dati
% numericke vijednosti za x kad ovaj sustav pokusamo rijesiti:

x = A\b

b1 = A*x % znamo da Ax nije jednko b. Izracunajmo koliko je Ax i nazovimo taj vektor b1
r = b1 - b % "rezidual": razilika izmedju b i b1. MATLAB nam je dao takav x, da je rezidual najmanji moguci

% teorija nam kaze da je r (rezidual) ortogonalan prema stupcima matrice A
% provjera:
r'*A

% primjer b)  "overdetermined system"

a1 = [2;0;1];
a2 = [-1; 2; 2];
a3 = a1+a2;
a4 = 0.5*a1+1.2*a2;

A = [a1, a2, a3, a4]
b = ones(3,1)

x = A\b

b1 = A*x
r = b1-b

% teorija nam kaze da je r (rezidual) ortogonalan prema stupcima matrice A
% provjera:
r'*A


%%% 3) A je matrica n x m gdje je n<m, broj redaka < broj stupaca

% eng. "underdetermined system"
% u ovom slucaju broj nepoznanica je veci nego broj jednandzbi
% MATLAB ce dati jedno ("partikularno") rjesenje
% skup svih rjesenja dobiva se linearnom kombinacijom patikularnog rjesenja
% i vektora iz baze prostora jezgre matrice (eng. "kernel" space ili "null" space)

A = [2 -1 4 2; 1 2 3 4]
b = [1; 2]

x = A\b % partikularno rjesenje

Z = null(A) % jezgra matrice A; dimezija matrice Z u ovom slucaju je 4 x 2
            % za svaki stupac iz Z, vrijedi da je A*(stupac iz Z)=0 (vektor s nulama)

A*Z(:, 1)
A*Z(:, 2)

% svaka linearna kombinacija partikularnog rjesenja x 
% i stupaca od Z je rjesenje pocetne jednadzbe
% PROVJERA
A*(x+2*Z(:,1)-5*Z(:,2))   % =b
b



%% SVOJSTVENE VRIJEDNOSTI I SVOJSTVENI VEKTORI

% vidjeti: help eig

help eig

M = randn(5,5)
[V, D] = eig(M)

N = [1 4 -2;3 2 5;-2 -4 2];

p = poly(N) % definira karakteristicni polinom matrice N (vidjeti: help poly)

roots(p) % nultocke karakteristicnog polinoma
eig(N) % svojstvene vrijednosti 


prod(eig(N)) % umnozak sv. vrijednosti jednak je determinanti
det(N)

sum(eig(N)) % zbroj sv. vrijednosti jednak je tragu matrice
trace(N)



%% VEKTORSKE I MATRICNE NORME

%     For matrices...
%       NORM(X) is the 2-norm of X.
%       NORM(X,2) is the same as NORM(X).
%       NORM(X,1) is the 1-norm of X.
%       NORM(X,inf) is the infinity norm of X.
%       NORM(X,'fro') is the Frobenius norm of X.
%       NORM(X,P) is available for matrix X only if P is 1, 2, inf or 'fro'.
%  
%     For vectors...
%       NORM(V,P) = sum(abs(V).^P)^(1/P).
%       NORM(V) = norm(V,2).
%       NORM(V,inf) = max(abs(V)).
%       NORM(V,-inf) = min(abs(V)).


%%% PRIMJER 1 %%%

v = [1;1]
norm(v)  % ili norm(v,2) = euklidska norma
sqrt(2)

%%% PRIMJER 2 %%%
% Treba odrediti jedinicni vektor (vektor duljine 1 u euklidskoj normi)
% paralelan s vektorom ("u smjeru vektora") [1; 2; 4]

v = [1; 2; 4]
vJedinicni = v/norm(v, 2)